from ._canvas import Canvas3D
from ._node import Node
from ._truss import Truss


class Model:
    def __init__(self):
        self._parent = None
        self._nodes = dict()
        self._elements = dict()

    def add_node(self, key, x, y, z, support=''):
        """Add a new node to the model.

        Parameters
        ----------
        key : str
            The unique identifier of the node.
        x : float
            The location of the node in x direction.
        y : float
            The location of the node in y direction.
        z : float
            The location of the node in z direction.
        support : str
            The directions in which a support permits displacements.

        Returns
        -------
        Node
            The new node.

        """
        node = Node(key, x, y, z)
        if 'x' in support:
            node.x.is_active = False
        if 'y' in support:
            node.y.is_active = False
        if 'z' in support:
            node.z.is_active = False
        self._nodes[key] = node
        return node

    def node(self, key):
        """Get a node by a given key.

        Parameters
        ----------
        key : str
            The unique identifier of the node.

        Returns
        -------
        Node
            The node with the given key.

        """
        if isinstance(key, Node):
            key = key.key
        node = self._nodes.get(key, None)
        if node is None:
            raise RuntimeError()
        return node

    def add_truss(self, key, node_a, node_b):
        """Add a new truss element to the model.

        Parameters
        ----------
        key : str
            The unique identifier of the element.
        node_a : Node or str
            The start node or the key of the node.
        node_b : Node or str
            The end node or the key of the node.

        Returns
        -------
        Truss
            The new truss element.

        """
        node_a = self.node(node_a)
        node_b = self.node(node_b)
        truss = Truss(key, node_a, node_b)
        self._elements[key] = truss
        return truss

    def show(self):
        """Shows a visualization of the model.
        """
        canvas = Canvas3D()
        for element in self._elements.values():
            element.draw(canvas)
        canvas.show()



class Truss:
    def __init__(self, key, node_a, node_b):
        self.key = key
        self.node_a = node_a
        self.node_b = node_b

    def draw(self, canvas):
        location_a = self.node_a.location
        location_b = self.node_b.location

        canvas.line(location_a, location_b, text=f'Truss {self.key}')



import numpy as np
import plotly.graph_objects as go


class Canvas3D:
    def __init__(self):
        self._fig = go.Figure()
        self._min = np.array([float('inf')] * 3, float)
        self._max = np.array([float('-inf')] * 3, float)

    def line(self, a, b, color='black', text=None):
        x, y, z = np.asarray([a, b], float).T

        self._fig.add_scatter3d(
            x=x,
            y=y,
            z=z,
            mode='lines',
            name=text,
            line=dict(
                width=2,
                color=color,
            ),
        )

        box_min = np.minimum(a, b)
        box_max = np.maximum(a, b)

        self._min = np.minimum(box_min, self._min)
        self._max = np.maximum(box_max, self._max)

    def show(self):
        mid = (self._max + self._min) * 0.5
        size = max(self._max - self._min)

        box_min = mid - size / 2
        box_max = mid + size / 2

        self._fig.update_layout(
            scene=dict(
                xaxis=dict(range=[box_min[0], box_max[0]]),
                yaxis=dict(range=[box_min[1], box_max[1]]),
                zaxis=dict(range=[box_min[2], box_max[2]]),
            ),
        )

        self._fig.show()



from setuptools import setup

setup(
   name='nfem',
   version='0.1',
   description='',
   author='Thomas Oberbichler',
   author_email='thomas.oberbichler@gmail.com',
   packages=['nfem'],
   install_requires=['numpy', 'plotly'],
)



class Dof:
    def __init__(self, value, is_active=True):
        self.initial_value = float(value)
        self.value = float(value)
        self.is_active = is_active
        self.external_force = 0.0

    def __float__(self):
        return self.value



from ._dof import Dof
import numpy as np


class Node:
    def __init__(self, key, x, y, z):
        self.key = key
        self.x = Dof(x)
        self.y = Dof(y)
        self.z = Dof(z)

    @property
    def location(self):
        return np.array([self.x, self.y, self.z], float)
